VDU	= $0300
SCAP	= $0020
LINE	= SCAP+01
WORK	= LINE+02 
SCRA	= $0400		; memory addresses for the screen 
SCRB	= $0500 
SCRC	= $0600 
SCRD	= $0700 
CRTA	= $0800		; 6845/6545 CRT controller 
CRTB	= $0801

	.org	$300
	
CHATS:
	LDY SCAP	; character to screen 
	CMP #$20 
	BCC CTL		; all control characters 
	CMP #$7F 
	BEQ DELETE
TOSCRN:
	JSR WRCH 
	INY
	CPY #$28 
	BCC VDUB	; automatic scroll when line filled 
FILLED:
	JSR SCROLL
VDUA:
	LDY #$00 
VDUB:
	JSR CALCN 
	STY SCAP 
	LDY #$0F	; rewrite cursor position 
	STY CRTA
	LDY WORK
	STY CRTB 
	LDY #$0E 
	STY CRTA 
	LDY WORK+02 
	STY CRTB
VDUC:
	RTS 
DELETE:
	DEY 
	BMI VDUC	; refuse to delete before line start 
	LDA #' '	; write in a blank 
	JSR WRCH 
	LDA #$7F 
	BNE VDUB
CTL:
	CMP #$0D	; carriage return? 
	BEQ VDUA 
	CMP #$0A	; line feed? 
	BEQ SCROL 
	CMP #$0C	; form feed? 
	BEQ CLEARS 
	BNE TOSCRN
SCROL:
	JSR SCROLL	; vscroll screen and rewrite cursor
	LDY SCAP
	BCS VDUB
CLEARS:
	PHA		; clear entire buffer
	LDY #$00
	LDA #' '
CLEAR:
	STA SCRA,Y
	STA SCRB,Y
	STA SCRC,Y
	STA SCRD,Y
	INY
	BNE CLEAR
	STY SCAP
	LDY #$0F
SETCRT:
	STY CRTA	; set up all the CRT parameters
	LDA CRTTAB,Y
	STA CRTB
	DEY
	BPL SETCRT
	LDA #$C0
	STA LINE
	LDA #$07
	STA LINE+01 
	PLA
	RTS 
SCROLL:
	PHP		; scroll subroutine 
	PHA
	CLD 
	LDY #$28 
	JSR CALCN 
	LDA WORK 
	STA LINE 
	LDA WORK+02 
	STA LINE+01 
	LDY #$0D
	STY CRTA
	LDA LINE
 	SEC
	SBC #$C0
	STA CRTB
	DEY
	STY CRTA
	LDA WORK+02
	SBC #$03
	STA CRTB
	LDY #$27
	LDA #' '
CLEARL:
	JSR WRCH
	DEY
	BPL CLEARL
	PLA
	PLP
	RTS
CALCN:
	PHP		; do calculation to make sure that the
	PHA		; processor and CRT controller agree on
	CLD		; position of screen r :
	CLC
	TYA
 	ADC LINE
 	STA WORK
 	LDA LINE+01
 	ADC #$00
 	STA WORK+02
 	AND #$07
 	ORA #$04
 	STA WORK+01
	PLA
	PLP
	RTS 
WRCH:
	JSR CALCN
	STY WORK+02
	LDY #$00
	STA (WORK),Y
	LDY WORK+02
	RTS 
CRTTAB:
	.byte $3F ; total number of characters per line,
	.byte $28 ; 40 characters displayed
	.byte $33 ; position of horizontal sync
	.byte $05 ; width in m S of horizontal sync pulse
	.byte $1E ; total number of character row
	.byte $02 ; additional no. of lines for 312 total
	.byte $19 ; 25 displayed character rows
	.byte $1B ; position of vertical sync pulse
	.byte $00 ; set non-interlace mode
	.byte $09 ; set 10 lines per character row
	.byte $68 ; slow blink cursor from line 9
	.byte $09 ; to line 10
	.byte $04 ; high address of VDU ram
	.byte $00 ; low address of VDU ram
	.byte $07 ; high address of initial cursor position
	.byte $C0 ; low address of initial cursor position 
