;Acorn Mini-Disassembler
;The mini-Disassembler provides a formatted listing of 25 lines 
; of 6502 program instructions in machine code presented in Hexadecimal.
;
;The start address of the program to be disassembled is entered 
; into the m address of the Acorn Monitor using the m key and the
; disassembler program is then executed from 0200 using the g key in the normal way.
; A formatted 25 lines of instructions will be displayed on the screen and the program ends in a jump back to the monitor. The m address is stepped forwards so re- running the program will display the next 25 program lines.
;MINDIS ACORN 6502 Assembler 

MINDIS	= $0200 
MOD	= $0000 
COUNT	= $000E 
VDU	= $0300 
RESTRT	= $FF04

	.org	$200

START:
	LDA #$18	; disassemble 25 lines 
	STA COUNT
	CLD 
	LDA #$0C	; start with a form-feed 
	JSR VDU
MAIN:
	LDA #$0D	; carriage return/line feed for each line 
	JSR VDU 
	LDA #$0A 
	JSR VDU 
	LDA MOD+01	; display current address 
	JSR SPBYTE 
	LDA MOD 
	JSR BYTOUT 
	LDY #$00 
	LDX #$01	; x will be the byte count of the opcode 
	LDA (MOD),Y	; fetch opcode, find it's no. of bytes 
	CMP #$20	; 'jsr' is an anomaly and is done first 
	BEQ CBYTE 
	AND #$9F 
	BEQ ABYTE	; binary 0XX00000 is 1 byte 
	AND #$1D 
	CMP #$19 
	BEQ CBYTE	; binary XXX110X1 is 3 bytes 
	AND #$0D 
	CMP #$08
	BEQ ABYTE	; binary XXXXX0X0 (now) is 1 byte 
	AND #$0C
	CMP #$0C
	BEQ CBYTE	; binary XXXX11 XX is 3 bytes
 	DEX		; all others are 2 byte
CBYTE:
	INX
 	INX
ABYTE:
	LDY #$00
	LDA (MOD),Y
	JSR SPBYTE
	INC MOD		; increment 
	BNE NOINC 
	INC MOD +01 
NOINC:
	DEX		; print all bytes required 
	BNE ABYTE
	DEC COUNT 
	BPL MAIN	; finished the 25 lines
GETOUT:
	JMP RESTRT
SPBYTE:
	PHA		; print a space and then the byte
	LDA #' '
	JSR VDU 
	PLA 
BYTOUT:
	PHA		; print a byte 
	LSR A
	LSR A 
	LSR A 
	LSR A 
	JSR DIGOUT 
	PLA
DIGOUT:
	AND #$0F	; print the bottom Hex digit in A 
	ORA #$30 
	CMP #$3A 
	BCC	PUT 
	ADC #$06 
PUT:
	JMP	VDU 
